#!/usr/bin/python3

import yaml
import subprocess
import getpass
import sys

DB_CREDS_FILE = '/opt/cyops/configs/database/db_config.yml'
YAML_UTILITY_FILE = '/opt/cyops/configs/scripts/confUtil.py'


def get_value(d, u):
    for k, v in d.items():
        if isinstance(v, dict) or isinstance(v, list):
            ret_val = get_value(d.get(k, {}), u)
            if ret_val != 0:
                return ret_val
        else:
            if k == u:
                return d[k]
    return 0


def restart_service_for_password_change():
    print('Restarting All Service')
    subprocess.check_output('csadm services --restart', shell=True)

    print('Clearing API Cache For Password Change')
    subprocess.check_output('sudo -u nginx php /opt/cyops-api/app/console cache:clear --env=prod --no-interaction',
                            shell=True)


with open(DB_CREDS_FILE, 'r') as outfile:
    config = yaml.load(outfile)
    pg_user = (get_value(config, 'pg_user'))
    pg_host = (get_value(config, 'pg_host'))

if pg_host in ['localhost', '127.0.0.1']:
    new_postgres_password = getpass.getpass('Please enter a password of your choice for the Postgres Database: ')
    confirm_new_postgres_password = getpass.getpass('Please confirm password for the Postgres Database: ')
else:
    new_postgres_password = getpass.getpass(
        'Please enter the password that you have set for your remote PostgresDB instance: ')
    confirm_new_postgres_password = getpass.getpass('Please confirm password for your remote PostgresDB instance: ')


if not (str(new_postgres_password)):
    print("Blank Postgres Password provided")
    sys.exit(1)

if new_postgres_password != confirm_new_postgres_password:
    print("Postgres Password do not match")
    sys.exit(1)
 

# Postgres Password Change
print('Changing Postgres Password')
if pg_host in ['localhost', '127.0.0.1']:
    pg_pwd = 'sudo -i -u postgres psql -U postgres -c "ALTER USER ' + pg_user + ' WITH ENCRYPTED PASSWORD \'' + str(new_postgres_password) + '\';"'
    changePostgersPasswordStatus = subprocess.check_output(pg_pwd, shell=True)
else:
    print('Postgres Password Changed. Please put the same password at postgres hosted at %s' % (pg_host))
    changePostgersPasswordStatus = 'Success'

# Adding PostgresDB New Password to File
print('Writing Encrypted Postgres Password to File')
if not ('exception' or 'error' or 'failed') in str(changePostgersPasswordStatus):
    pg_encrypted_password = subprocess.check_output(
        'python3 /opt/cyops/configs/scripts/manage_passwords.py --encrypt ' + str(new_postgres_password),
        shell=True).strip().decode()
    write_pg_pwd = 'python3 ' + YAML_UTILITY_FILE + ' -f ' + DB_CREDS_FILE + ' -k pg_password -v \'' + str(
        pg_encrypted_password) + '\''
    subprocess.check_output(write_pg_pwd, shell=True)
else:
    print("PostgresDB Password Change Failed with following error: %s" % str(changePostgersPasswordStatus))
    restart_service_for_password_change()
    sys.exit(2)

restart_service_for_password_change()
