#!/bin/bash

# This script changes the hostname, nodename for rabbitmq,
# delete DB and re-create the DB of rabbitmq.

env_file="/etc/profile"
rabbitmq_db="/var/lib/rabbitmq/mnesia/"
rabbitmq_cookie="/var/lib/rabbitmq/.erlang.cookie"
rabbitmq_config="/opt/cyops-rabbitmq/configs/rabbitmq/rabbitmq_configs.sh"

delete_db_rabbitmq()
{
    echo "Deleting rabbitmq db."
    systemctl stop rabbitmq-server
    # This will remove the rabbitmq database.
    rm -rf  $rabbitmq_db
    rm -rf  $rabbitmq_cookie
}

change_hostname()
{
    echo "Changing the hostname."
    # The following condition checks if HOSTNAME exist or not in the file /etc/profile.
    grep -r "export HOSTNAME" $env_file
    if [ $? = 0 ]; then
       host_var=`grep -r "export HOSTNAME" $env_file | awk '{print $2}'`
       sed -i  "s/$host_var/$new_host_name/g" $env_file
    else
       echo "export HOSTNAME=$host_name" >> $env_file
    fi
    . $env_file
    hostnamectl set-hostname $host_name
}

create_db_rabbitmq()
{
    echo "Creating the DB for rabbitmq."	
    systemctl start rabbitmq-server
    # The following script creates the rabbitmq user, vhost, etc.
    if [ -f $rabbitmq_config ]; then
        sh $rabbitmq_config
    fi
    if [ $? != 0 ];then
        echo "File $rabbitmq_config not found"
        exit 1
    fi
}

restart_services()
{
    echo "Restarting all services." 
    # Need to restart the services after hostname change and create rabbitmq db.
    csadm services --restart
}

err_msg()
{
    echo "Usage: sh change_hostname.sh -h <hostname>"
}

while getopts h: name
do
	case $name in
		h)
			host_name="$OPTARG"
			new_host_name="HOSTNAME=$host_name"
		;;
   		?)
			err_msg
			exit
        ;;
    esac
done;

if [ ! -z $host_name ]; then
    # Change hostname and rename the rabbitmq node. 
    delete_db_rabbitmq
    change_hostname
    create_db_rabbitmq
    restart_services
else
    # Rename the rabbitmq node name only.
    delete_db_rabbitmq
    create_db_rabbitmq
    restart_services
fi
